/****************************************/
/* Cool Herders                         */
/* A demented muse by Brendan Wiese     */
/* http://starbow.org                   */
/* Programmed by Mike Brent             */
/* http://tursi.yiffco.com              */
/* sheep.c                              */
/****************************************/

#include <stdio.h>
#include <malloc.h>
#include <kallisti/libk.h>
#include <kallisti/timer.h>
#include <kallisti/abi/fs.h>
#include <kallisti/abi/ta.h>
#include <kallisti/abi/spu.h>
#include <kallisti/abi/maple.h>
#include <kallisti/abi/video.h>
#include <kallisti/abi/thread.h>
#include <kallisti/abi/sndsrv_abi.h>
#include "sprite.h"
#include "cool.h"
#include "rand.h"
#include "collide.h"
#include "levels.h"
#include "sheep.h"

HerdStruct sheep[MAX_SHEEP];

// Set up the sheep on this level
void initSheep() {
	int idx, idx2;
	float t1, t2;

	printf("Initializing Sheep\n");

	for (idx=0; idx<MAX_SHEEP; idx++) {
		sheep[idx].spr.txr_addr=txr_sheep;
		sheep[idx].spr.tilenumber=0;
		sheep[idx].spr.alpha=1.0;
		t1=(float)(10.0*GRIDSIZE+PLAYFIELDXOFF);
		t2=(float)(7.0*GRIDSIZE+PLAYFIELDYOFF);
		sheep[idx].spr.x=t1;
		sheep[idx].spr.y=t2;
		sheep[idx].spr.z=516.0;
		sheep[idx].spr.xd=1;
		sheep[idx].spr.yd=0;
		sheep[idx].spr.is3D=false;
		sheep[idx].spr.ta_style=TA_TRANSLUCENT;
		sheep[idx].type=2;	// active
		sheep[idx].destx=0;
		sheep[idx].desty=0;
		sheep[idx].score=0;
		sheep[idx].stuntime=0;
		sheep[idx].animframe=0;
		sheep[idx].maxframes=SHEEPFRAMES;
		sheep[idx].sheep=0;
		sheep[idx].range=240;	// start with 4 seconds of invincibility
		sheep[idx].trap=0;
		for (idx2=0; idx2<HISTORY; idx2++) {
			sheep[idx].oldx[idx2]=sheep[idx].spr.x;
			sheep[idx].oldy[idx2]=sheep[idx].spr.y;
		}
		sheep[idx].oldidx=0;
	}
}

// Move the sheep. This is a lot like the computer Herder movement
void moveSheep() {
	int idx, miscflag, tmp, speed, mask;
	int currentx, currenty, gridx, gridy;
	int xstep, ystep, desiredx, desiredy;
	int origx, origy, loop;
	
	for (idx=0; idx<MAX_SHEEP; idx++) {
		if (sheep[idx].type>0) {
			for (loop=0; loop<sheep[idx].type; loop++) {
				origx=sheep[idx].spr.x;
				origy=sheep[idx].spr.y;

				if (sheep[idx].stuntime==0) {
					// Check - if we didn't move, choose a new destination
					if ((sheep[idx].oldx[sheep[idx].oldidx>0?sheep[idx].oldidx-1:HISTORY-1]==sheep[idx].spr.x)&&(sheep[idx].oldy[sheep[idx].oldidx>0?sheep[idx].oldidx-1:HISTORY-1]==sheep[idx].spr.y)) {
						sheep[idx].destx=0;
					}

					updateoldpos(&sheep[idx]);

					if ((sheep[idx].destx==0)||(sheep[idx].desty==0)) {
						// new target - it doesn't have to be a legal destination,
						// as we'll pick a new one as soon as we get stuck :)
						sheep[idx].destx=(rand()%20) * GRIDSIZE;
						sheep[idx].desty=(rand()%15) * GRIDSIZE;
					}

					sheep[idx].spr.xd=sheep[idx].destx-sheep[idx].spr.x;
					sheep[idx].spr.yd=sheep[idx].desty-sheep[idx].spr.y;
					if (nGameFrames%4 == 0) {
						sheep[idx].animframe++;
					}
					miscflag=1;

					if (sheep[idx].type>1) {
						sheep[idx].spr.alpha=0.5f;
						if (sheep[idx].range>0) {
							sheep[idx].range--;
						}
						if (sheep[idx].range==0) {
							sheep[idx].type=1;		// back to normal next frame
						} else {
							if (sheep[idx].range<60) {
								sheep[idx].type=2;	// slow down in preparation...
							}
						}
					} else {
						sheep[idx].spr.alpha=1.0f;
					}

					if (sheep[idx].spr.xd>0) {
						sheep[idx].spr.x+=min(sheep[idx].spr.xd, SHEEPSPEED);
						sheep[idx].spr.xd=1;
					} else {
						sheep[idx].spr.x-=min(-sheep[idx].spr.xd, SHEEPSPEED);
						sheep[idx].spr.xd=-1;
					}
					if (sheep[idx].spr.yd>0) {
						sheep[idx].spr.y+=min(sheep[idx].spr.yd, SHEEPSPEED);
						sheep[idx].spr.yd=1;
					} else {
						sheep[idx].spr.y-=min(-sheep[idx].spr.yd, SHEEPSPEED);
						sheep[idx].spr.yd=-1;
					}

					if (sheep[idx].animframe >= sheep[idx].maxframes) {
						sheep[idx].animframe=0;
					}
					
					// Check legality of new position, and shift towards legal spot
					// Save current position to reduce indirections!
					// No action if no movement
					if (miscflag) {
						currentx=sheep[idx].spr.x;
						currenty=sheep[idx].spr.y;
						gridx=((currentx-PLAYFIELDXOFF)+GRIDSIZE/2)/GRIDSIZE;
						gridy=((currenty-PLAYFIELDYOFF)+GRIDSIZE/2)/GRIDSIZE;
						xstep=(int)sheep[idx].spr.xd;
						ystep=(int)sheep[idx].spr.yd;
						desiredx=currentx;
						desiredy=currenty;

						if (sheep[idx].type==1) {
							mask=0;
						} else {
							mask=TYPE_WALL;	// only care about walls
						}
						
						if (xstep!=0) {
							// We're walking left or right.
							if (checkblock(currentx, origy, mask)) {
								// we're walking into a wall
								desiredx=origx;
							}
						} else {
							desiredx=gridx*GRIDSIZE+PLAYFIELDXOFF;
						}
						if (ystep!=0) {
							// We're walking up or down
							if (checkblock(origx, currenty, mask)) {
								desiredy=origy;
							}
						} else {
							desiredy=gridy*GRIDSIZE+PLAYFIELDYOFF;
						}
						if ((desiredx!=origx)&&(desiredy!=origy)) {
							// we're still trying to go diagonally, and both dirs are legal
							// We need to choose just one. We'd prefer the direction we
							// did not move last frame
							if (sheep[idx].oldx[sheep[idx].oldidx>0?sheep[idx].oldidx-1:HISTORY-1]!=origx) {
								// we WERE going horizontal, so switch to vertical
								desiredx=origx;
							} else {
								// we WERE either going nowhere or vertical, take horizontal
								desiredy=origy;
							}
						}
						
						// Now adjust towards desired. We move at SHEEPSPEED to override movement
						// But not more pixels than are required :)
						if (desiredx>currentx) {
							sheep[idx].spr.x+=min(desiredx-currentx, SHEEPSPEED+1);
						}
						if (desiredx<currentx) {
							sheep[idx].spr.x-=min(currentx-desiredx, SHEEPSPEED+1);
						}
						if (desiredy>currenty) {
							sheep[idx].spr.y+=min(desiredy-currenty, SHEEPSPEED+1);
						}
						if (desiredy<currenty) {
							sheep[idx].spr.y-=min(currenty-desiredy, SHEEPSPEED+1);
						}
						// Done grid adjustment - make final legality check
						if (checkblock(sheep[idx].spr.x, sheep[idx].spr.y, mask)) {
							sheep[idx].spr.x=origx;
							sheep[idx].spr.y=origy;
						}
						// Finally, set the correct frame
						sheep[idx].spr.tilenumber=SHEEPDOWN;
						if (origy>sheep[idx].spr.y) sheep[idx].spr.tilenumber=SHEEPUP;
						if (origx>sheep[idx].spr.x) sheep[idx].spr.tilenumber=SHEEPLEFT;
						if (origx<sheep[idx].spr.x) sheep[idx].spr.tilenumber=SHEEPRIGHT;
		
						sheep[idx].spr.tilenumber+=sheep[idx].animframe*5;
					}
				} else {	// stunned
					sheep[idx].stuntime--;
					tmp=(nGameFrames/10)%4;
					switch (tmp) {
					case 0:
						sheep[idx].spr.tilenumber=SHEEPUP; break;
					case 1:
						sheep[idx].spr.tilenumber=SHEEPDOWN; break;
					case 2:
						sheep[idx].spr.tilenumber=SHEEPLEFT; break;
					case 3:
						sheep[idx].spr.tilenumber=SHEEPRIGHT; break;
					}
				}

				SortSprite(&sheep[idx].spr);
			}
		}
	}
}
